import { android as androidApp } from "tns-core-modules/application";
import { View } from "tns-core-modules/ui/core/view";

// Alias the actual package to an internal variable
const GlassboxBridge: any = com.clarisite.mobile.Glassbox;
const StartupSettingsBuilder = com.clarisite.mobile.StartupSettings.StartupSettingsBuilder;

function toHashMap(obj : any) {
	const map = new java.util.HashMap();
	if(obj === undefined) return map;

    for (var property in obj) {
        if (obj.hasOwnProperty(property)) {
			const value : any = obj[property];
            if (value != null) {
                switch (typeof value) {
                    case 'object':
                        map.put(property, toHashMap(value));
                        break;
                    case 'boolean':
                        map.put(property, java.lang.Boolean.valueOf(String(value)));
                        break;
                    case 'number':
                        if (Number(obj[property]) === obj[property] && obj[property] % 1 === 0) 
							map.put(property, java.lang.Long.valueOf(String(value)));
                        else
                            map.put(property, java.lang.Double.valueOf(String(value)));
                        break;
                    case 'string':
                        map.put(property, String(obj[property]));
                        break;
                }
            }
        }
    };

    return map;
}

export const start = function(reportUrl: string, appid: string, isInternalMode = false) {
		if(GlassboxBridge === undefined) {
			console.warn("glassbox-nativescript plugin wasn't installed properly. native api does not exist");
			return;
		}

		const context = androidApp.context;
		//GlassboxBridge.start(context);
		if(context !== undefined) {
			let builder: com.clarisite.mobile.StartupSettings.StartupSettingsBuilder = 
			StartupSettingsBuilder.aSettingsBuilder().
				withReportUrl(reportUrl).
				withAppId(appid).
				withApplicationCtx(context);
			
			if(isInternalMode) {
				builder.setInternalConfigurationPolicy();
			}

			GlassboxBridge.start(builder.build());
		} 
};

export const stop = function() {
	if(GlassboxBridge === undefined) {
		console.warn("glassbox-nativescript plugin wasn't installed properly. native api does not exist");
		return;
	}

	GlassboxBridge.stop();
}

export const startScreen = function(screenName: string) {
	if(GlassboxBridge !== undefined) {
		console.warn("glassbox-nativescript plugin wasn't installed properly. native api does not exist");
		return;
	}

	GlassboxBridge.startScreen(screenName);
}

export const endScreen = function() {
	if(GlassboxBridge === undefined) {
		console.warn("glassbox-nativescript plugin wasn't installed properly. native api does not exist");
		return;
	}

	GlassboxBridge.endScreen();
}

export const reportEvent = function(eventName: string, parameters: any) {
	if(GlassboxBridge === undefined) {
		console.warn("glassbox-nativescript plugin wasn't installed properly. native api does not exist");
		return;
	}

	GlassboxBridge.reportCustomEvent(eventName, toHashMap(parameters));
}

export const setViewAsSensitive = function(view: View) {
	if(GlassboxBridge === undefined) {
		console.warn("glassbox-nativescript plugin wasn't installed properly. native api does not exist");
		return;
	}

	GlassboxBridge.setViewAsSensitive(view.nativeView.nativeViewProtected);
}