
#import "RNStoryboard.h"
#import <React/RCTUIManager.h>
#import <React/RCTBaseTextInputView.h>

@implementation RNStoryboard

@synthesize bridge = _bridge;

- (dispatch_queue_t)methodQueue
{
    return dispatch_get_main_queue();
}

RCT_EXPORT_METHOD(start: (NSString *) serverUrl appid:(NSString *) appid) {
    dispatch_async(dispatch_get_main_queue(), ^{
        [Storyboard setSessionDelegate:self];
        GLAStartupSettings *settings = [[GLAStartupSettings alloc] init];
        settings.reportUrl = serverUrl;
        settings.appId = appid;
        settings.hybridMode = YES;
        NSError * err = nil;
        [Storyboard setSDKType:@"ReactNative"];
        [Storyboard start:settings error:&err];
    });
}

RCT_EXPORT_METHOD(stop) {
    dispatch_async(dispatch_get_main_queue(), ^{
        [Storyboard stop];
    });
}

RCT_EXPORT_METHOD(reportEvent: (NSString *) eventName properties:(NSDictionary * ) properties) {
    dispatch_async(dispatch_get_main_queue(), ^{
        [Storyboard reportEvent:eventName parameters:properties];
    });
}

RCT_EXPORT_METHOD(setViewAsSensitive:(nonnull NSNumber *) viewTag maskingFlags:(NSDictionary *)maskingFlags) {
    if (!viewTag) {
        return;
    }
    
    dispatch_async(dispatch_get_main_queue(), ^{
        UIView* view = [self.bridge.uiManager viewForReactTag:viewTag];
        if (!view) {
            return;
        }
        
        // for TextInput retrieve the RCTTextField stored in backedTextInputView
        if([view isKindOfClass:RCTBaseTextInputView.class] &&
           [(RCTBaseTextInputView *)view respondsToSelector:@selector(backedTextInputView)]) {
           view = [(RCTBaseTextInputView *)view backedTextInputView];
        }
        
        GLAVisibilityFlags *flags = [GLAVisibilityFlags flagsFromDictionary:maskingFlags];
        [Storyboard hideSensitiveView:view visibilityFlags:flags];
    });
}

RCT_EXPORT_METHOD(disableViewHiding:(nonnull NSNumber *) viewTag) {
    if (!viewTag) {
        return;
    }
    
    dispatch_async(dispatch_get_main_queue(), ^{
        UIView* view = [self.bridge.uiManager viewForReactTag:viewTag];
        if (!view) {
            return;
        }
        
        // for TextInput retrieve the RCTTextField stored in backedTextInputView
        if([view isKindOfClass:RCTBaseTextInputView.class] &&
           [(RCTBaseTextInputView *)view respondsToSelector:@selector(backedTextInputView)]) {
           view = [(RCTBaseTextInputView *)view backedTextInputView];
        }
        
        [Storyboard disableViewHiding:view];
    });
}

RCT_EXPORT_METHOD(setViewTag:(nonnull NSNumber *) viewTag tag:(NSString *) tag) {
    if (!viewTag) {
        return;
    }
    
    dispatch_async(dispatch_get_main_queue(), ^{
        UIView* view = [self.bridge.uiManager viewForReactTag:viewTag];
        if (!view) {
            return;
        }
        
        [view performSelector:@selector(setGlassboxTag:) withObject:tag];
    });
}

RCT_EXPORT_METHOD(startScreen:(NSString *)screenName) {
    if(!screenName) {
        return;
    }
    
    dispatch_async(dispatch_get_main_queue(), ^{
        [Storyboard startScreen:screenName];
    });
}

RCT_EXPORT_METHOD(endScreen) {
    dispatch_async(dispatch_get_main_queue(), ^{
        [Storyboard endScreen];
    });
}

RCT_EXPORT_METHOD(trackView:(nonnull NSNumber *) viewTag) {
    //NOT NEEDED IN IOS
}


- (void)sessionStartedWithConfiguration:(NSDictionary *)configuration {
#if DEBUG
    
#endif
}

- (void)sessionFailedWithError:(NSError *)error {
#if DEBUG
    NSLog(@"GB Session failed with error %@", error);
#endif
}

- (void)sessionExcludedWithReason:(NSString *)reason withConfiguration:(NSDictionary *)configuration {
#if DEBUG
    NSLog(@"GB Session excluded due to reason %@ configuration %@", reason, configuration);
#endif
}

RCT_EXPORT_MODULE(Storyboard)

@end
